#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Finds forms definitions
"""

import logging

from django import forms
from django.conf import settings
from django.core import validators
from django.core.exceptions import PermissionDenied
from django.forms.formsets import formset_factory
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from ishtar_common.models import valid_id, valid_ids, get_current_profile, \
    SpatialReferenceSystem
from archaeological_operations.models import Period, ArchaeologicalSite, \
    RelationType as OpeRelationType
from archaeological_context_records.models import DatingType, DatingQuality, \
    ContextRecord
import models

from ishtar_common.forms import FormSet, FloatField, \
    get_form_selection, reverse_lazy, TableSelect, get_now, FinalForm, \
    ManageOldType

from ishtar_common.forms_common import get_town_field, SourceSelect
from ishtar_common.utils import convert_coordinates_to_point
from ishtar_common import widgets
from archaeological_operations.widgets import OAWidget

from archaeological_warehouse.models import Warehouse

from archaeological_finds.forms_treatments import TreatmentSelect, \
    TreatmentFormSelection, BaseTreatmentForm, TreatmentModifyForm, \
    AdministrativeActTreatmentForm, TreatmentFormFileChoice, \
    TreatmentDeletionForm, TreatmentFileSelect, TreatmentFileFormSelection, \
    TreatmentFileForm, TreatmentFileModifyForm, TreatmentFileDeletionForm, \
    AdministrativeActTreatmentFormSelection, \
    AdministrativeActTreatmentModifForm, \
    AdministrativeActTreatmentFileForm, \
    AdministrativeActTreatmentFileFormSelection, \
    AdministrativeActTreatmentFileModifForm, SourceTreatmentFormSelection, \
    SourceTreatmentFileFormSelection, TreatmentSourceFormSelection, \
    TreatmentFileSourceFormSelection

__all__ = [
    'TreatmentSelect', 'TreatmentFormSelection', 'BaseTreatmentForm',
    'TreatmentModifyForm', 'AdministrativeActTreatmentForm',
    'TreatmentFormFileChoice', 'TreatmentDeletionForm',
    'AdministrativeActTreatmentModifForm', 'TreatmentFileSelect',
    'TreatmentFileFormSelection', 'TreatmentFileForm',
    'TreatmentFileModifyForm', 'TreatmentFileDeletionForm',
    'AdministrativeActTreatmentFileForm',
    'AdministrativeActTreatmentFileFormSelection',
    'AdministrativeActTreatmentFormSelection',
    'AdministrativeActTreatmentFileModifForm', 'SourceTreatmentFormSelection',
    'SourceTreatmentFileFormSelection', 'TreatmentSourceFormSelection',
    'TreatmentFileSourceFormSelection',
    'RecordFormSelection', 'FindForm', 'DateForm', 'DatingFormSet',
    'FindSelect', 'FindFormSelection', 'FindFormSelectionWarehouseModule',
    'MultipleFindFormSelection', 'MultipleFindFormSelectionWarehouseModule',
    'FindMultipleFormSelection', 'check_form', 'check_exist', 'check_not_exist',
    'check_value', 'check_type_field', 'check_type_not_field',
    'check_treatment', 'ResultFindForm', 'ResultFindFormSet',
    'FindDeletionForm', 'UpstreamFindFormSelection', 'SourceFindFormSelection',
    'FindSourceSelect', 'FindSourceFormSelection', 'NewFindBasketForm',
    'SelectFindBasketForm', 'DeleteFindBasketForm', 'FindBasketAddItemForm']

logger = logging.getLogger(__name__)


class RecordFormSelection(forms.Form):
    form_label = _("Context record")
    base_models = ['get_first_base_find']
    associated_models = {'get_first_base_find__context_record': ContextRecord}
    get_first_base_find__context_record = forms.IntegerField(
        label=_(u"Context record"), required=False,
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-contextrecord'),
            associated_model=ContextRecord),
        validators=[valid_id(ContextRecord)])

    def __init__(self, *args, **kwargs):
        super(RecordFormSelection, self).__init__(*args, **kwargs)
        # get the current operation and restrict search to it
        cr_pk = None
        if 'data' in kwargs and kwargs['data']:
            cr_pk = kwargs['data'].get(
                'get_first_base_find__context_record')
        if not cr_pk and 'initial' in kwargs and kwargs['initial']:
            cr_pk = kwargs['initial'].get(
                'get_first_base_find__context_record')
        if not cr_pk:
            return
        try:
            cr = ContextRecord.objects.get(pk=cr_pk)
        except ContextRecord.DoesNotExist:
            return
        widget = self.fields['get_first_base_find__context_record'].widget
        widget.source = unicode(widget.source) + "?operation__pk={}".format(
            cr.operation.pk)


class FindForm(ManageOldType, forms.Form):
    file_upload = True
    form_label = _("Find")
    base_models = ['get_first_base_find', 'object_type', 'material_type',
                   'preservation_to_consider', 'integritie',
                   'remarkabilitie']
    associated_models = {'material_type': models.MaterialType,
                         'conservatory_state': models.ConservatoryState,
                         'object_type': models.ObjectType,
                         'preservation_to_consider': models.PreservationType,
                         'integritie': models.IntegrityType,
                         'get_first_base_find__batch': models.BatchType,
                         'remarkabilitie': models.RemarkabilityType,
                         'get_first_base_find__spatial_reference_system':
                         SpatialReferenceSystem}
    label = forms.CharField(
        label=_(u"Free ID"),
        validators=[validators.MaxLengthValidator(60)])
    previous_id = forms.CharField(label=_("Previous ID"), required=False)
    description = forms.CharField(label=_("Description"),
                                  widget=forms.Textarea, required=False)
    get_first_base_find__batch = forms.ChoiceField(
        label=_(u"Batch/object"), choices=[],
        required=False)
    is_complete = forms.NullBooleanField(label=_(u"Is complete?"),
                                         required=False)
    material_type = widgets.Select2MultipleField(
        label=_(u"Material type"), required=False
    )
    conservatory_state = forms.ChoiceField(label=_(u"Conservatory state"),
                                           choices=[], required=False)
    conservatory_comment = forms.CharField(
        label=_(u"Conservatory comment"), required=False,
        widget=forms.Textarea)
    object_type = widgets.Select2MultipleField(
        label=_(u"Object types"), required=False,
    )
    preservation_to_consider = forms.MultipleChoiceField(
        label=_(u"Preservation type"), choices=[],
        widget=widgets.Select2Multiple, required=False)
    integritie = forms.MultipleChoiceField(
        label=_(u"Integrity / interest"), choices=[],
        widget=widgets.Select2Multiple, required=False)
    remarkabilitie = forms.MultipleChoiceField(
        label=_(u"Remarkability"), choices=[],
        widget=widgets.Select2Multiple, required=False)
    get_first_base_find__topographic_localisation = forms.CharField(
        label=_(u"Point of topographic reference"),
        required=False, max_length=120
    )
    get_first_base_find__x = forms.FloatField(label=_(u"X"), required=False)
    get_first_base_find__y = forms.FloatField(label=_(u"Y"), required=False)
    get_first_base_find__z = forms.FloatField(label=_(u"Z"), required=False)
    get_first_base_find__spatial_reference_system = \
        forms.ChoiceField(label=_(u"Spatial Reference System"), required=False,
                          choices=[])
    get_first_base_find__estimated_error_x = \
        forms.FloatField(label=_(u"Estimated error for X"), required=False)
    get_first_base_find__estimated_error_y = \
        forms.FloatField(label=_(u"Estimated error for Y"), required=False)
    get_first_base_find__estimated_error_z = \
        forms.FloatField(label=_(u"Estimated error for Z"), required=False)
    length = FloatField(label=_(u"Length (cm)"), required=False)
    width = FloatField(label=_(u"Width (cm)"), required=False)
    height = FloatField(label=_(u"Height (cm)"), required=False)
    diameter = FloatField(label=_(u"Diameter (cm)"), required=False)
    thickness = FloatField(label=_(u"Thickness (cm)"), required=False)
    volume = FloatField(label=_(u"Volume (l)"), required=False)
    weight = FloatField(label=_(u"Weight (g)"), required=False)
    dimensions_comment = forms.CharField(
        label=_(u"Dimensions comment"), required=False, widget=forms.Textarea)
    find_number = forms.IntegerField(label=_(u"Find number"), required=False)
    min_number_of_individuals = forms.IntegerField(
        label=_(u"Minimum number of individuals (MNI)"), required=False)
    mark = forms.CharField(label=_(u"Mark"), required=False)
    checked = forms.ChoiceField(label=_(u"Check"))
    check_date = forms.DateField(
        initial=get_now, label=_(u"Check date"), widget=widgets.JQueryDate)
    comment = forms.CharField(label=_(u"Comment"), required=False,
                              widget=forms.Textarea)
    dating_comment = forms.CharField(
        label=_(u"Comment on dating"), required=False, widget=forms.Textarea)
    estimated_value = FloatField(label=_(u"Estimated value"), required=False)
    image = forms.ImageField(
        label=_(u"Image"), help_text=mark_safe(
            _(u"<p>Heavy images are resized to: %(width)dx%(height)d "
              u"(ratio is preserved).</p>") % {
                'width': settings.IMAGE_MAX_SIZE[0],
                'height': settings.IMAGE_MAX_SIZE[1]}),
        max_length=255, required=False, widget=widgets.ImageFileInput())

    def __init__(self, *args, **kwargs):
        super(FindForm, self).__init__(*args, **kwargs)
        if not get_current_profile().mapping:
            for k in ['get_first_base_find__x', 'get_first_base_find__y',
                      'get_first_base_find__z',
                      'get_first_base_find__estimated_error_x',
                      'get_first_base_find__estimated_error_y',
                      'get_first_base_find__estimated_error_z',
                      'get_first_base_find__spatial_reference_system',]:
                self.fields.pop(k)
        else:
            srs = 'get_first_base_find__spatial_reference_system'
            self.fields[srs].choices = \
                SpatialReferenceSystem.get_types(
                    initial=self.init_data.get(srs))
            self.fields[srs].help_text = \
                SpatialReferenceSystem.get_help()
        self.fields['checked'].choices = models.CHECK_CHOICES
        self.fields['material_type'].choices = models.MaterialType.get_types(
            initial=self.init_data.get('material_type'),
            empty_first=False
        )
        self.fields['material_type'].help_text = models.MaterialType.get_help()

        self.fields['object_type'].choices = models.ObjectType.get_types(
            initial=self.init_data.get('object_type'),
            empty_first=False
        )
        self.fields['object_type'].help_text = models.ObjectType.get_help()

        self.fields['get_first_base_find__batch'].choices = \
            models.BatchType.get_types(
                initial=self.init_data.get('get_first_base_find__batch'))
        self.fields['get_first_base_find__batch'].help_text = \
            models.BatchType.get_help()

        self.fields['conservatory_state'].choices = \
            models.ConservatoryState.get_types(
                initial=self.init_data.get('conservatory_state'))
        self.fields['conservatory_state'].help_text = \
            models.ConservatoryState.get_help()

        self.fields['preservation_to_consider'].choices = \
            models.PreservationType.get_types(
                empty_first=False,
                initial=self.init_data.get('preservation_to_consider'))
        self.fields['preservation_to_consider'].help_text = \
            models.PreservationType.get_help()
        self.fields['integritie'].choices = \
            models.IntegrityType.get_types(
                empty_first=False,
                initial=self.init_data.get('integritie'))
        self.fields['integritie'].help_text = \
            models.IntegrityType.get_help()
        self.fields['remarkabilitie'].choices = \
            models.RemarkabilityType.get_types(
                empty_first=False,
                initial=self.init_data.get('remarkabilitie'))
        self.fields['remarkabilitie'].help_text = \
            models.RemarkabilityType.get_help()
        self.fields['estimated_value'].label = u"{} ({})".format(
            unicode(self.fields['estimated_value'].label),
            get_current_profile().currency)

    def clean(self):
        if not get_current_profile().mapping:
            return self.cleaned_data
        x = self.cleaned_data.get('get_first_base_find__x', None)
        y = self.cleaned_data.get('get_first_base_find__y', None)
        s = 'get_first_base_find__spatial_reference_system'
        srs = self.cleaned_data.get(s, None)
        if srs:
            try:
                srs = SpatialReferenceSystem.objects.get(pk=srs)
            except SpatialReferenceSystem.DoesNotExist:
                srs = None
        if x and y and not srs:
            raise forms.ValidationError(
                _(u"You should at least provide X, Y and the spatial "
                  u"reference system used."))
        if x and y and srs:
            try:
                convert_coordinates_to_point(
                    x, y, self.cleaned_data.get('get_first_base_find__z', None),
                    srs.srid)
            except forms.ValidationError as e:
                raise forms.ValidationError(
                    unicode(_(u"Coordinates are not relevant for the spatial "
                              u"reference system used: {}.")).format(e))
        return self.cleaned_data


class DateForm(ManageOldType, forms.Form):
    form_label = _("Dating")
    base_model = 'dating'
    associated_models = {'dating_type': DatingType,
                         'quality': DatingQuality,
                         'period': Period}
    period = forms.ChoiceField(label=_("Period"), choices=[])
    start_date = forms.IntegerField(label=_(u"Start date"),
                                    required=False)
    end_date = forms.IntegerField(label=_(u"End date"), required=False)
    quality = forms.ChoiceField(label=_("Quality"), required=False,
                                choices=[])
    dating_type = forms.ChoiceField(label=_("Dating type"),
                                    required=False, choices=[])
    precise_dating = forms.CharField(label=_("Precise dating"),
                                     required=False)

    def __init__(self, *args, **kwargs):
        super(DateForm, self).__init__(*args, **kwargs)
        self.fields['dating_type'].choices = DatingType.get_types(
            initial=self.init_data.get('dating_type'))
        self.fields['dating_type'].help_text = DatingType.get_help()
        self.fields['period'].choices = Period.get_types(
            initial=self.init_data.get('period'))
        self.fields['period'].help_text = Period.get_help()
        self.fields['quality'].choices = DatingQuality.get_types(
            initial=self.init_data.get('quality'))
        self.fields['quality'].help_text = DatingQuality.get_help()


DatingFormSet = formset_factory(DateForm, can_delete=True,
                                formset=FormSet)
DatingFormSet.form_label = _("Dating")


class FindSelect(TableSelect):
    base_finds__cache_short_id = forms.CharField(label=_(u"Short ID"))
    base_finds__cache_complete_id = forms.CharField(label=_(u"Complete ID"))
    label = forms.CharField(label=_(u"Free ID"))
    base_finds__context_record__parcel__town = get_town_field()
    base_finds__context_record__operation__year = forms.IntegerField(
        label=_(u"Year"))
    base_finds__context_record__operation__operation_code = forms.IntegerField(
        label=_(u"Operation's number (index by year)"))
    base_finds__context_record__operation__code_patriarche = \
        forms.IntegerField(
            label=_(u"Code PATRIARCHE"),
            widget=OAWidget
        )
    archaeological_sites = forms.IntegerField(
        label=_("Archaeological site"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-archaeologicalsite'),
            associated_model=ArchaeologicalSite),
        validators=[valid_id(ArchaeologicalSite)])
    ope_relation_types = forms.MultipleChoiceField(
        label=_(u"Search within related operations"), choices=[],
        widget=widgets.CheckboxSelectMultiple)
    datings__period = forms.ChoiceField(label=_(u"Period"), choices=[])
    material_types = forms.ChoiceField(label=_(u"Material type"), choices=[])
    object_types = forms.ChoiceField(label=_(u"Object type"), choices=[])
    preservation_to_considers = forms.ChoiceField(
        choices=[], label=_(u"Preservation type"))
    conservatory_state = forms.ChoiceField(label=_(u"Conservatory state"),
                                           choices=[])
    integrities = forms.ChoiceField(label=_(u"Integrity / interest"),
                                    choices=[])
    remarkabilities = forms.ChoiceField(label=_(u"Remarkability"),
                                        choices=[])
    base_finds__find__description = forms.CharField(label=_(u"Description"))
    base_finds__batch = forms.ChoiceField(
        label=_(u"Batch/object"), choices=[])
    checked = forms.ChoiceField(label=_("Check"))
    image__isnull = forms.NullBooleanField(label=_(u"Has an image?"))

    def __init__(self, *args, **kwargs):
        super(FindSelect, self).__init__(*args, **kwargs)
        self.fields['datings__period'].choices = Period.get_types()
        self.fields['datings__period'].help_text = Period.get_help()
        self.fields['material_types'].choices = \
            models.MaterialType.get_types()
        self.fields['material_types'].help_text = \
            models.MaterialType.get_help()
        self.fields['conservatory_state'].choices = \
            models.ConservatoryState.get_types()
        self.fields['conservatory_state'].help_text = \
            models.ConservatoryState.get_help()

        self.fields['base_finds__batch'].choices = \
            models.BatchType.get_types()
        self.fields['base_finds__batch'].help_text = \
            models.BatchType.get_help()

        self.fields['object_types'].choices = \
            models.ObjectType.get_types()
        self.fields['checked'].choices = \
            [('', '--')] + list(models.CHECK_CHOICES)
        self.fields['preservation_to_considers'].choices = \
            models.PreservationType.get_types()
        self.fields['preservation_to_considers'].help_text = \
            models.PreservationType.get_help()
        self.fields['integrities'].choices = \
            models.IntegrityType.get_types()
        self.fields['integrities'].help_text = \
            models.IntegrityType.get_help()
        self.fields['remarkabilities'].choices = \
            models.RemarkabilityType.get_types()
        self.fields['remarkabilities'].help_text = \
            models.RemarkabilityType.get_help()
        self.fields['ope_relation_types'].choices = OpeRelationType.get_types(
            empty_first=False)

    def get_input_ids(self):
        ids = super(FindSelect, self).get_input_ids()
        ids.pop(ids.index('ope_relation_types'))
        for idx, c in enumerate(self.fields['ope_relation_types'].choices):
            ids.append('ope_relation_types_{}'.format(idx))
        return ids


class FindSelectWarehouseModule(FindSelect):
    container__location = forms.IntegerField(
        label=_(u"Warehouse (location)"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-warehouse'),
            associated_model=Warehouse),
        validators=[valid_id(Warehouse)])
    container__responsible = forms.IntegerField(
        label=_(u"Warehouse (responsible)"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-warehouse'),
            associated_model=Warehouse),
        validators=[valid_id(Warehouse)])
    container__index = forms.IntegerField(label=_(u"Container ID"))
    container__reference = forms.IntegerField(label=_(u"Container ref."))


class FindFormSelection(forms.Form):
    form_label = _("Find search")
    associated_models = {'pk': models.Find}
    currents = {'pk': models.Find}
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-find'),
            FindSelect, models.Find,
            source_full=reverse_lazy('get-find-full')),
        validators=[valid_id(models.Find)])


class FindFormSelectionWarehouseModule(FindFormSelection):
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-find'),
            FindSelectWarehouseModule, models.Find,
            source_full=reverse_lazy('get-find-full')),
        validators=[valid_id(models.Find)])


class MultipleFindFormSelection(forms.Form):
    form_label = _("Find search")
    associated_models = {'pk': models.Find}
    currents = {'pk': models.Find}
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-find'),
            FindSelect, models.Find,
            multiple_select=True,
            source_full=reverse_lazy('get-find-full')),
        validators=[valid_id(models.Find)])


class MultipleFindFormSelectionWarehouseModule(MultipleFindFormSelection):
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-find'),
            FindSelectWarehouseModule, models.Find,
            multiple_select=True,
            source_full=reverse_lazy('get-find-full')),
        validators=[valid_id(models.Find)])


class FindMultipleFormSelection(forms.Form):
    form_label = _(u"Upstream finds")
    associated_models = {'finds': models.Find}
    associated_labels = {'finds': _(u"Finds")}
    # using FindSelectWarehouseModule because this form is only used with
    # the warehouse module activated
    finds = forms.CharField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-find'), FindSelectWarehouseModule, models.Find,
            multiple=True, multiple_cols=[2, 3, 4]),
        validators=[valid_ids(models.Find)])

    def clean(self):
        if 'finds' not in self.cleaned_data or not self.cleaned_data['finds']:
            raise forms.ValidationError(_(u"You should at least select one "
                                          u"archaeological find."))
        return self.cleaned_data


def check_form(wizard, form_name, key):
    request = wizard.request
    storage = wizard.storage
    if storage.prefix not in request.session or \
            'step_data' not in request.session[storage.prefix] or \
            form_name not in request.session[storage.prefix]['step_data'] or \
                            form_name + '-' + key not in \
            request.session[storage.prefix]['step_data'][form_name]:
        return False
    return True


def check_exist(form_name, key):
    def func(self):
        request = self.request
        storage = self.storage
        if not check_form(self, form_name, key):
            return False
        try:
            val = request.session[storage.prefix][
                'step_data'][form_name][form_name + '-' + key]
            if val and len(val) > 0:
                val = val[0]
            return bool(val)
        except ValueError:
            return False
    return func


def check_not_exist(form_name, key):
    def func(self):
        return not check_exist(form_name, key)(self)
    return func


def check_value(form_name, key, value):
    def func(self):
        request = self.request
        storage = self.storage
        if not check_form(self, form_name, key):
            return False
        try:
            val = request.session[storage.prefix][
                'step_data'][form_name][form_name + '-' + key]
            if val and len(val) > 0:
                val = val[0]
            return unicode(val) == unicode(value)
        except ValueError:
            return False
    return func


def check_type_field(form_name, key, model, field):
    def func(self):
        request = self.request
        storage = self.storage
        if not check_form(self, form_name, key):
            return False
        try:
            val = model.objects.get(pk=request.session[storage.prefix][
                'step_data'][form_name][form_name + '-' + key][0])
            return bool(getattr(val, field))
        except (ValueError, model.DoesNotExist):
            return False
    return func


def check_type_not_field(form_name, key, model, field):
    def func(self):
        return not check_type_field(form_name, key, model, field)(self)
    return func


def check_treatment(form_name, type_key, type_list=[], not_type_list=[]):
    type_list = [models.TreatmentType.objects.get(txt_idx=tpe).pk
                 for tpe in type_list]
    not_type_list = [models.TreatmentType.objects.get(txt_idx=tpe).pk
                     for tpe in not_type_list]

    def func(self):
        request = self.request
        storage = self.storage
        if not check_form(self, form_name, type_key):
            return False
        try:
            tpe = request.session[storage.prefix][
                'step_data'][form_name][form_name + '-' + type_key]
            if not tpe:
                return False
            type = int(tpe[0])
            return (not type_list or type in type_list) \
                and type not in not_type_list
        except ValueError:
            return False
    return func


class ResultFindForm(ManageOldType, forms.Form):
    form_label = _(u"Resulting find")
    associated_models = {'material_type': models.MaterialType}
    label = forms.CharField(
        label=_(u"Free ID"),
        validators=[validators.MaxLengthValidator(60)])
    description = forms.CharField(label=_(u"Precise description"),
                                  widget=forms.Textarea)
    material_type = forms.ChoiceField(label=_(u"Material type"), choices=[])
    volume = forms.IntegerField(label=_(u"Volume (l)"))
    weight = forms.IntegerField(label=_(u"Weight (g)"))
    find_number = forms.IntegerField(label=_(u"Find number"))

    def __init__(self, *args, **kwargs):
        super(ResultFindForm, self).__init__(*args, **kwargs)
        self.fields['material_type'].choices = models.MaterialType.get_types(
            initial=self.init_data.get('material_type'))
        self.fields['material_type'].help_text = models.MaterialType.get_help()

ResultFindFormSet = formset_factory(ResultFindForm, can_delete=True,
                                    formset=FormSet)
ResultFindFormSet.form_label = _(u"Resulting finds")


class FindDeletionForm(FinalForm):
    confirm_msg = " "
    confirm_end_msg = _(u"Would you like to delete this find?")


class UpstreamFindFormSelection(FindFormSelection):
    form_label = _(u"Upstream find")

    def __init__(self, *args, **kwargs):
        super(UpstreamFindFormSelection, self).__init__(*args, **kwargs)
        self.fields['pk'].required = True
        self.fields['resulting_pk'] = self.fields.pop('pk')


##############################################
# Source management for archaeological finds #
##############################################

SourceFindFormSelection = get_form_selection(
    'SourceFindFormSelection', _(u"Archaeological find search"), 'find',
    models.Find, FindSelect, 'get-find',
    _(u"You should select an archaeological find."))


class FindSourceSelect(SourceSelect):
    find__base_finds__context_record__operation__year = forms.IntegerField(
        label=_(u"Year of the operation"))
    find__base_finds__context_record__operation__operation_code = \
        forms.IntegerField(label=_(u"Numeric reference"))
    if settings.COUNTRY == 'fr':
        find__base_finds__context_record__operation__code_patriarche = \
            forms.IntegerField(
                widget=OAWidget,
                label="Code PATRIARCHE")
    find__datings__period = forms.ChoiceField(
        label=_(u"Period of the archaeological find"), choices=[])
    find__material_type = forms.ChoiceField(
        label=_("Material type of the archaeological find"), choices=[])
    find__description = forms.CharField(
        label=_(u"Description of the archaeological find"))

    def __init__(self, *args, **kwargs):
        super(FindSourceSelect, self).__init__(*args, **kwargs)
        self.fields['find__datings__period'].choices = Period.get_types()
        self.fields['find__datings__period'].help_text = Period.get_help()
        self.fields['find__material_type'].choices = \
            models.MaterialType.get_types()
        self.fields['find__material_type'].help_text = \
            models.MaterialType.get_help()

FindSourceFormSelection = get_form_selection(
    'FindSourceFormSelection', _(u"Documentation search"), 'pk',
    models.FindSource, FindSourceSelect, 'get-findsource',
    _(u"You should select a document."),
    get_full_url='get-findsource-full')


class NewFindBasketForm(forms.ModelForm):
    class Meta:
        model = models.FindBasket
        fields = ('label', 'comment')

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')
        super(NewFindBasketForm, self).__init__(*args, **kwargs)

    def clean(self):
        q = models.FindBasket.objects.filter(user=self.user,
                                             label=self.cleaned_data['label'])
        if q.count():
            raise forms.ValidationError(_(u"Another basket already exists with "
                                          u"this name."))
        return self.cleaned_data

    def save(self, commit=True):
        self.instance.user = self.user
        return super(NewFindBasketForm, self).save(commit)


class SelectFindBasketForm(forms.Form):
    form_label = _(u"Basket")
    associated_models = {'basket': models.FindBasket}
    need_user_for_initialization = True

    basket = forms.ChoiceField(label=_(u"Basket"), required=True, choices=[])

    def __init__(self, *args, **kwargs):
        self.user = None
        if 'user' in kwargs:
            self.user = kwargs.pop('user')
        super(SelectFindBasketForm, self).__init__(*args, **kwargs)
        if not self.user:
            return
        self.fields['basket'].choices = [('', '--')] + [
            (b.pk, unicode(b))
            for b in models.FindBasket.objects.filter(user=self.user)]


class DeleteFindBasketForm(SelectFindBasketForm):
    def save(self):
        try:
            models.FindBasket.objects.get(pk=self.cleaned_data['basket'],
                                          user=self.user).delete()
        except models.FindBasket.DoesNotExist:
            # something strange... TODO: log it
            pass
        return


class FindBasketAddItemForm(forms.Form):
    basket_id = forms.IntegerField(required=True)
    item_id = forms.IntegerField(required=True)

    def save(self, user):
        try:
            basket = models.FindBasket.objects.get(
                pk=self.cleaned_data['basket_id'], user=user.ishtaruser)
            item = models.Find.objects.get(
                pk=self.cleaned_data['item_id'])
        except models.FindBasket.DoesNotExist or\
                models.Find.DoesNotExist:
            # something strange... TODO: log it
            raise PermissionDenied
        # check rights
        if not user.is_superuser and \
                not user.ishtaruser.has_right('change_find') and \
                not (user.ishtaruser.has_right('change_own_find')
                     and item.is_own(user)):
            raise PermissionDenied
        basket.items.add(item)
        return basket



from ishtar_common.models import Town


dup_nb = 0
pdl = []
DPTS = ('44', '49', '53', '72', '85')

for dpt in DPTS:
    for town in Town.objects.filter(numero_insee__startswith=dpt):
        pdl.append(town.pk)
        for dup in Town.objects.filter(name=town.name).exclude(pk=town.pk):
            not_dup = False
            for d in DPTS:
                if dup.numero_insee.startswith(d):
                    not_dup = True
            if not_dup:
                continue
            for item in dup.file_main.all():
                item.main_town = town
                p = item.save()
            for item in dup.parcels.all():
                item.main_town = town
                p = item.save()
            for item in dup.file.all():
                item.towns.remove(dup)
                item.towns.add(town)
            for item in dup.operations.all():
                item.towns.remove(dup)
                item.towns.add(town)
            dup_nb += 1
            dup.delete()


print("{} items cleaned".format(dup_nb))

strange = []
for town in Town.objects.exclude(pk__in=pdl):
    if (town.file_main.count() or town.parcels.count() or town.file.count() or
            town.operations.count()):
        strange.append((town, town.file_main.count(), town.parcels.count(),
                        town.file.count(), town.operations.count()))
        continue
    town.delete()


print('* Problems with:')
for t in strange:
    print("{}: \n\t* {} ville principale dossier\n\t* {} parcelles\n\t* {} "
          "villes pour"
          " dossier\n\t* {} ville pour operation".format(t[0], t[1], t[2],
                                                         t[3], t[4]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2014 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf.urls.defaults import *

from archaeological_files_pdl import views

urlpatterns = patterns(
    '',
    url(r'file_creation/(?P<step>.+)?$',
        views.file_creation_wizard, name='file_creation'),
    url(r'file_modification/(?P<step>.+)?$',
        views.file_modification_wizard, name='file_modification'),
    url(r'townplanning-edit/$',
        views.TownPlanningCreate.as_view(),
        name='townplanning_create'),
    url(r'townplanning-edit/(?P<pk>\d+)$',
        views.TownPlanningEdit.as_view(),
        name='townplanning_edit'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.utils.translation import ugettext_lazy as _

from ishtar_common.menu_base import SectionItem, MenuItem

import models

# be carreful: each access_controls must be relevant with check_rights in urls

MENU_SECTIONS = [
    (5, SectionItem('admin', _(u"Administration"),
     childs=[
        SectionItem(
            'account', _(u"Account"),
            childs=[MenuItem('account_management', _(u"Addition/modification"),
                             model=models.IshtarUser,
                             access_controls=['add_ishtaruser', ]),
                    MenuItem('account_deletion', _(u"Deletion"),
                             model=models.IshtarUser,
                             access_controls=['add_ishtaruser', ]), ]),
        MenuItem('admin-globalvar', _(u"Global variables"),
                 model=models.GlobalVar,
                 access_controls=['add_globalvar', ]),
    ])
    ),
    (10, SectionItem('administration', _(u"Directory"),
     childs=[
        SectionItem(
            'person', _(u"Person"),
            childs=[
                MenuItem(
                    'person_search', _(u"Search"),
                    model=models.Person,
                    access_controls=['add_person']),
                MenuItem(
                    'person_creation', _(u"Creation"),
                    model=models.Person,
                    access_controls=['add_person']),
                MenuItem(
                    'person_modification', _(u"Modification"),
                    model=models.Person,
                    access_controls=['change_person', 'change_own_person']),
                MenuItem(
                    'person-merge', _(u"Automatic merge"),
                    model=models.Person,
                    access_controls=['merge_person']),
                MenuItem(
                    'person-manual-merge', _(u"Manual merge"),
                    model=models.Person,
                    access_controls=['merge_person']),
                MenuItem(
                    'person_deletion', _(u"Deletion"),
                    model=models.Person,
                    access_controls=['change_person', 'change_own_person']),
            ]),
        SectionItem(
            'organization', _(u"Organization"),
            childs=[
                MenuItem(
                    'organization_search', _(u"Search"),
                    model=models.Organization,
                    access_controls=['add_organization',
                                     'add_own_organization']),
                MenuItem(
                    'organization_creation', _(u"Creation"),
                    model=models.Organization,
                    access_controls=['add_organization',
                                     'add_own_organization']),
                MenuItem(
                    'organization_modification', _(u"Modification"),
                    model=models.Organization,
                    access_controls=['change_organization',
                                     'change_own_organization']),
                MenuItem(
                    'organization-merge', _(u"Automatic merge"),
                    model=models.Organization,
                    access_controls=['merge_organization']),
                MenuItem(
                    'orga-manual-merge', _(u"Manual merge"),
                    model=models.Organization,
                    access_controls=['merge_organization']),
                MenuItem(
                    'organization_deletion', _(u"Deletion"),
                    model=models.Organization,
                    access_controls=['change_organization',
                                     'change_own_organization']),
            ]),
    ])
    ),
    (15, SectionItem(
        'imports', _(u"Imports"),
        childs=[
            MenuItem(
                'import-new', _(u"New import"),
                model=models.Import,
                access_controls=['change_import']),
            MenuItem(
                'import-list', _(u"Current imports"),
                model=models.Import,
                access_controls=['change_import']),
            MenuItem(
                'import-list-old', _(u"Old imports"),
                model=models.Import,
                access_controls=['change_import']),
        ])),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Admin description
"""
import csv

from django.conf import settings
from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _, ugettext

from django import forms

import models


class ImportGenericForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    csv_file = forms.FileField(
        "CSV file", help_text="Only unicode encoding is managed - convert your"
        " file first")


def gen_import_generic(self, request, queryset):
    form = None

    if 'apply' in request.POST:
        form = ImportGenericForm(request.POST, request.FILES)

        if form.is_valid():
            csv_file = request.FILES['csv_file']
            reader = csv.reader(csv_file)
            idx = 0
            for row in reader:
                slug = slugify(row[0])
                if self.model.objects.filter(txt_idx=slug).count():
                    continue
                obj, c = self.model.objects.get_or_create(
                    label=row[0], txt_idx=slug)
                if c:
                    idx += 1
            self.message_user(request, "Successfully added %d new items." % (
                idx))
            return HttpResponseRedirect(request.get_full_path())
    if not form:
        form = ImportGenericForm(
            initial={'_selected_action':
                     request.POST.getlist(admin.ACTION_CHECKBOX_NAME)})
    return render_to_response(
        'admin/import_from_csv.html', {'csv_form': form},
        context_instance=RequestContext(request))

gen_import_generic.short_description = "Import from a CSV file"


def export_as_csv_action(description=_(u"Export selected as CSV file"),
                         fields=None, exclude=None, header=True):
    """
    This function returns an export csv action
    'fields' and 'exclude' work like in django ModelForm
    'header' is whether or not to output the column names as the first row
    """
    def export_as_csv(modeladmin, request, queryset):
        """
        Generic csv export admin action.
        based on http://djangosnippets.org/snippets/1697/
        """
        opts = modeladmin.model._meta
        field_names = set([field.name for field in opts.fields])
        if fields:
            fieldset = set(fields)
            field_names = field_names & fieldset
        elif exclude:
            excludeset = set(exclude)
            field_names = field_names - excludeset

        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=%s.csv' % \
            unicode(opts).replace('.', '_')

        writer = csv.writer(response)
        if header:
            writer.writerow(list(field_names))
        for obj in queryset:
            writer.writerow([
                unicode(getattr(obj, field)).encode("utf-8", "replace")
                for field in field_names])
        return response
    export_as_csv.short_description = description
    return export_as_csv


class HistorizedObjectAdmin(admin.ModelAdmin):
    readonly_fields = ('history_modifier',)

    def save_model(self, request, obj, form, change):
        obj.history_modifier = request.user
        obj.save()


class MyGroupAdmin(GroupAdmin):
    class Media:
        css = {
            "all": ("media/admin.css",)
        }

admin.site.unregister(Group)
admin.site.register(Group, MyGroupAdmin)


class IshtarSiteProfileAdmin(admin.ModelAdmin):
    list_display = ('label', 'slug', 'active', 'files', 'context_record',
                    'find', 'warehouse')
    model = models.IshtarSiteProfile

admin.site.register(models.IshtarSiteProfile, IshtarSiteProfileAdmin)


class DepartmentAdmin(admin.ModelAdmin):
    list_display = ('number', 'label',)
    model = models.Department

admin.site.register(models.Department, DepartmentAdmin)


class OrganizationAdmin(HistorizedObjectAdmin):
    list_display = ('pk', 'name', 'organization_type')
    list_filter = ("organization_type",)
    search_fields = ('name',)
    exclude = ('merge_key', 'merge_exclusion', 'merge_candidate', )
    model = models.Organization

admin.site.register(models.Organization, OrganizationAdmin)


class PersonAdmin(HistorizedObjectAdmin):
    list_display = ('pk', 'name', 'surname', 'raw_name', 'email')
    list_filter = ("person_types",)
    search_fields = ('name', 'surname', 'email', 'raw_name')
    exclude = ('merge_key', 'merge_exclusion', 'merge_candidate', )
    model = models.Person

admin.site.register(models.Person, PersonAdmin)


class TownAdmin(admin.ModelAdmin):
    list_display = ['name', ]
    search_fields = ['name']
    if settings.COUNTRY == 'fr':
        list_display += ['numero_insee', 'departement', ]
        search_fields += ['numero_insee', 'departement__label', ]
        list_filter = ("departement",)
    model = models.Town

admin.site.register(models.Town, TownAdmin)


class AuthorAdmin(admin.ModelAdmin):
    list_display = ['person', 'author_type']
    list_filter = ("author_type",)
    model = models.Author

admin.site.register(models.Author, AuthorAdmin)


class PersonTypeAdmin(admin.ModelAdmin):
    list_display = ['label', 'txt_idx', 'available', 'comment']
    model = models.PersonType
    filter_vertical = ('groups',)

admin.site.register(models.PersonType, PersonTypeAdmin)


class GlobalVarAdmin(admin.ModelAdmin):
    list_display = ['slug', 'description', 'value']
admin.site.register(models.GlobalVar, GlobalVarAdmin)


class GeneralTypeAdmin(admin.ModelAdmin):
    list_display = ['label', 'txt_idx', 'available', 'comment']
    search_fields = ('label', 'txt_idx', 'comment',)
    actions = ['import_generic', export_as_csv_action()]
    import_generic = gen_import_generic


general_models = [models.OrganizationType, models.SourceType,
                  models.AuthorType, models.TitleType, models.Format,
                  models.SupportType]
for model in general_models:
    admin.site.register(model, GeneralTypeAdmin)


class ImporterDefaultValuesInline(admin.TabularInline):
    model = models.ImporterDefaultValues


class ImporterDefaultAdmin(admin.ModelAdmin):
    list_display = ('importer_type', 'target')
    model = models.ImporterDefault
    inlines = (ImporterDefaultValuesInline,)
admin.site.register(models.ImporterDefault, ImporterDefaultAdmin)


class ImporterTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'associated_models', 'is_template')
admin.site.register(models.ImporterType, ImporterTypeAdmin)


class RegexpAdmin(admin.ModelAdmin):
    list_display = ('name', 'description', "regexp")
admin.site.register(models.Regexp, RegexpAdmin)


class ImporterDuplicateFieldInline(admin.TabularInline):
    model = models.ImporterDuplicateField


class ImportTargetForm(forms.ModelForm):
    class Meta:
        model = models.ImportTarget
        exclude = []
        widgets = {
            'comment': forms.TextInput
        }


class ImportTargetInline(admin.TabularInline):
    model = models.ImportTarget
    extra = 1
    form = ImportTargetForm


class ImporterColumnAdmin(admin.ModelAdmin):
    list_display = ('label', 'importer_type', 'col_number', 'description',
                    'targets_lbl', 'duplicate_fields_lbl', 'required')
    list_filter = ('importer_type',)
    inlines = (ImportTargetInline, ImporterDuplicateFieldInline)
admin.site.register(models.ImporterColumn, ImporterColumnAdmin)


class ImporterModelAdmin(admin.ModelAdmin):
    list_display = ('name', 'klass')
    model = models.ImporterModel

admin.site.register(models.ImporterModel, ImporterModelAdmin)


class FormaterTypeAdmin(admin.ModelAdmin):
    list_display = ('formater_type', 'options')
admin.site.register(models.FormaterType, FormaterTypeAdmin)


class ImportAdmin(admin.ModelAdmin):
    list_display = ('name', 'importer_type', 'imported_file', 'user', 'state',
                    'creation_date')
admin.site.register(models.Import, ImportAdmin)


class TargetKeyAdmin(admin.ModelAdmin):
    list_display = ('target', 'importer_type', 'column_nb', 'key',
                    'value', 'is_set')
    list_filter = ("is_set", "target__column__importer_type")
    search_fields = ('target__target', 'value', 'key')
admin.site.register(models.TargetKey, TargetKeyAdmin)


class OperationTypeAdmin(GeneralTypeAdmin):
    list_display = GeneralTypeAdmin.list_display + ['order', 'preventive']
    model = models.OperationType

admin.site.register(models.OperationType, OperationTypeAdmin)


class SpatialReferenceSystemAdmin(GeneralTypeAdmin):
    list_display = GeneralTypeAdmin.list_display + ['order', 'srid']
    model = models.SpatialReferenceSystem
admin.site.register(models.SpatialReferenceSystem, SpatialReferenceSystemAdmin)


class IshtarUserAdmin(admin.ModelAdmin):
    readonly_fields = ('password',)

admin.site.register(models.IshtarUser, IshtarUserAdmin)


class ItemKeyAdmin(admin.ModelAdmin):
    list_display = ('content_type', 'key', 'content_object', 'importer')
    search_fields = ('key', )
admin.site.register(models.ItemKey, ItemKeyAdmin)


basic_models = [models.DocumentTemplate]
if settings.COUNTRY == 'fr':
    basic_models += [models.Arrondissement, models.Canton]

for model in basic_models:
    admin.site.register(model)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.shortcuts import render_to_response
from django.template import RequestContext

from ishtar_common.forms import reverse_lazy
from ishtar_common.wizards import Wizard, DeletionWizard
from archaeological_finds.wizards import TreatmentWizard

from archaeological_finds.models import Treatment, TreatmentType
import models


class PackagingWizard(TreatmentWizard):
    basket_step = 'base-packaging'

    def save_model(self, dct, m2m, whole_associated_models, form_list,
                   return_object):
        dct = self.get_extra_model(dct, form_list)
        obj = self.get_current_saved_object()
        dct['location'] = dct['container'].location
        items = None
        if 'items' in dct:
            items = dct.pop('items')
        if 'basket' in dct:
            if not items:
                items = dct.pop('basket')
            else:
                dct.pop('basket')
        treatment = Treatment(**dct)
        extra_args_for_new = {"container": dct['container']}
        treatment.save(items=items, user=self.request.user,
                       extra_args_for_new=extra_args_for_new)
        packaging = TreatmentType.objects.get(txt_idx='packaging')
        treatment.treatment_types.add(packaging)
        res = render_to_response('ishtar/wizard/wizard_done.html', {},
                                 context_instance=RequestContext(self.request))
        return return_object and (obj, res) or res


class WarehouseWizard(Wizard):
    model = models.Warehouse
    wizard_done_window = reverse_lazy('show-warehouse')


class WarehouseModificationWizard(Wizard):
    model = models.Warehouse
    modification = True
    wizard_done_window = reverse_lazy('show-warehouse')
    wizard_templates = {
        'divisions-warehouse_modification':
            'ishtar/wizard/wizard_warehouse_divisions.html',
    }

    def get_form_kwargs(self, step=None):
        kwargs = super(WarehouseModificationWizard, self).get_form_kwargs(step)
        if step == "divisions-warehouse_modification":
            current_warehouse = self.get_current_object()
            q = models.ContainerLocalisation.objects.filter(
                    division__warehouse=current_warehouse)
            if q.count():
                kwargs['readonly'] = True
        return kwargs


class WarehouseDeletionWizard(DeletionWizard):
    model = models.Warehouse


class ContainerWizard(Wizard):
    model = models.Container
    wizard_templates = {
        'localisation-container_creation':
            'ishtar/wizard/wizard_containerlocalisation.html',
        'localisation-container_modification':
            'ishtar/wizard/wizard_containerlocalisation.html',
    }
    ignore_init_steps = ['localisation']
    wizard_done_window = reverse_lazy('show-container')

    def get_form_kwargs(self, step=None):
        kwargs = super(ContainerWizard, self).get_form_kwargs(step)
        if step == 'localisation-' + self.url_name:
            container_pk = self.session_get_value(
                'selec-' + self.url_name, 'pk')
            q = models.Container.objects.filter(pk=container_pk)
            if q.count():
                kwargs['container'] = q.all()[0]
            warehouse_pk = self.session_get_value(
                'container-' + self.url_name, 'location')
            q = models.Warehouse.objects.filter(pk=warehouse_pk)
            if q.count():
                kwargs['warehouse'] = q.all()[0]
        return kwargs

    def done(self, form_list, **kwargs):
        """
        Save the localisation
        """
        super(ContainerWizard, self).done(form_list)
        dct = {}
        for idx, form in enumerate(form_list):
            if not form.is_valid():
                return self.render(form)
            container = self.get_current_object() or \
                hasattr(self, 'current_object') and self.current_object
            if container and form.prefix == 'localisation-' + self.url_name:
                for div_name in form.cleaned_data:
                    try:
                        division = models.WarehouseDivisionLink.objects.get(
                            pk=div_name.split('_')[-1],
                            warehouse=container.location
                        )  # check the localisation match with the container
                    except models.WarehouseDivisionLink.DoesNotExist:
                        return self.render(form)
                    localisation, created = \
                        models.ContainerLocalisation.objects.get_or_create(
                            container=container,
                            division=division
                        )
                    localisation.reference = form.cleaned_data[div_name]
                    localisation.save()
            dct = {'item': container}
            self.current_object = container
            # force evaluation of lazy urls
            dct['wizard_done_window'] = unicode(self.wizard_done_window)
        return render_to_response(
            self.wizard_done_template, dct,
            context_instance=RequestContext(self.request))


class ContainerModificationWizard(ContainerWizard):
    modification = True


class ContainerDeletionWizard(DeletionWizard):
    model = models.Container
    fields = ['container_type', 'reference', 'comment', 'location', 'index',
              'cached_label']
